@@ -66,10 +66,11 @@ guint slideshow_timer = 0;
 guint fullscreen_toolbar_timer = 0;
 guint fullscreen_pointer_timer = 0;
 
-guint x_start, y_start, x_max, y_max;
+gint x_start, y_start, x_max, y_max;
 gint pointer_x, pointer_y = 0;
 double xadj_start, yadj_start;
-guint zoom_timer_id;
+guint zoom_timer_id = 0;
+static gint current_rotation = 0;
 
 static void toggle_fullscreen ();
 
@@ -161,6 +162,7 @@ check_fullscreen_pointer ()
   }
   
   gdk_display_get_pointer (gdk_display_get_default (), NULL, &pointer_x, &pointer_y, NULL);
+  gtk_window_present(GTK_WINDOW(fullscreen_toggle_popup));
   
   return TRUE;
 }
@@ -177,7 +179,8 @@ pan_button_down (GtkWidget *w, GdkEventButton *b, GtkWidget *scrolled_window)
   x_max = (gdk_pixbuf_get_width (GDK_PIXBUF (scaled_image_pixbuf))) - (scrolled_window->allocation.width - 4);
   y_max = (gdk_pixbuf_get_height (GDK_PIXBUF (scaled_image_pixbuf))) - (scrolled_window->allocation.height - 4);
 
-  gdk_pointer_grab (w->window, 
+  if ((x_max > 0) && (y_max > 0))
+    gdk_pointer_grab (w->window, 
 		    FALSE, GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK,
 		    confine_pointer_to_window ? w->window : NULL, NULL, b->time);
 }
@@ -228,13 +231,14 @@ show_image (GtkWidget *widget, GList *image_filename)
   gint widget_width, widget_height;
   float width_ratio, height_ratio;
   float scale_width_ratio, scale_height_ratio;
+  GdkPixbuf *pixbuf = NULL;
 
   if (view_widget)
   	gtk_widget_hide (view_widget);
 
   image_filenames = image_filename;
   image_pixbuf = gdk_pixbuf_new_from_file ((gchar *) image_filename->data, NULL);
-
+    
   widget_width = main_scrolled_window->allocation.width - 5;
   widget_height = main_scrolled_window->allocation.height - 5;
   if (widget_width <= 0)
@@ -270,6 +274,15 @@ show_image (GtkWidget *widget, GList *image_filename)
                              (int)(widget_width * scale_width_ratio), 
                              (int)(widget_height * scale_height_ratio), 
                              GDK_INTERP_BILINEAR);
+  /* rotate image to current setting */
+  if (current_rotation)
+    {
+      pixbuf = image_tools_rotate (GDK_PIXBUF (scaled_image_pixbuf), 
+		                           current_rotation);
+      g_object_unref (scaled_image_pixbuf);
+      scaled_image_pixbuf = pixbuf;
+	}
+	
   gtk_image_set_from_pixbuf (GTK_IMAGE (image_widget), 
                              GDK_PIXBUF (scaled_image_pixbuf));
   
@@ -278,10 +291,10 @@ show_image (GtkWidget *widget, GList *image_filename)
       g_object_unref (image_pixbuf);
 	  image_pixbuf = NULL;
 	}
-  gtk_widget_show (tools_toolbar);
   gtk_widget_show (image_event_box);
   gtk_widget_show (image_widget);
   gtk_widget_show (main_scrolled_window);
+  gtk_widget_show (tools_toolbar);
 }
 
 static void
@@ -311,18 +324,19 @@ image_zoom_hyper ()
 }
 
 static void
-image_rotate () // BROKEN
+image_rotate () 
 {
   GdkPixbuf *pixbuf;
 
-  gtk_timeout_remove (zoom_timer_id);
-
-  pixbuf = image_tools_rotate (GDK_PIXBUF (scaled_image_pixbuf));
+  //gtk_timeout_remove (zoom_timer_id);
+  current_rotation++;
+  current_rotation = current_rotation % 4;
+  pixbuf = image_tools_rotate (GDK_PIXBUF (scaled_image_pixbuf), 1);
   g_object_unref (scaled_image_pixbuf);
   gtk_image_set_from_pixbuf (GTK_IMAGE (image_widget), GDK_PIXBUF (pixbuf));
   scaled_image_pixbuf = pixbuf;
-
-  zoom_timer_id = gtk_timeout_add (1000, image_zoom_hyper, scaled_image_pixbuf);
+	
+  //zoom_timer_id = gtk_timeout_add (1000, image_zoom_hyper, scaled_image_pixbuf);
 }
 
 static void
@@ -402,8 +416,8 @@ image_zoom_fit ()
 
   if (fullscreen_state == 1)
   {
-    widget_width = main_scrolled_window->allocation.width - 5;
-    widget_height = main_scrolled_window->allocation.height - 5;
+    widget_width = gdk_screen_width () - 5;
+    widget_height = gdk_screen_height () - 5;
   }
   else
  {
@@ -624,12 +638,15 @@ render_list_view ()
   struct stat file_stats;
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), 
+                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
   gtk_widget_size_request (scrolled_window, &scrolled_window_requisition);
 
   drawing_area = gtk_drawing_area_new ();
-  gtk_widget_set_size_request (GTK_WIDGET (drawing_area), scrolled_window_requisition.width, num_items * (MAX_ICON_SIZE + (MARGIN_Y * 2) + 1));
+  gtk_widget_set_size_request (GTK_WIDGET (drawing_area), 
+                               scrolled_window_requisition.width - 1, 
+                               num_items * (MAX_ICON_SIZE + (MARGIN_Y * 2) + 1));
 
   gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), drawing_area);
   gtk_widget_show_all (scrolled_window);
@@ -804,6 +821,7 @@ add_directory (gchar *directory)
   DIR *dir;
 
   loading_directory = 1;
+  current_rotation = 0;
   
   for (tl=g_list_first(image_filenames);tl;tl=g_list_next(tl))
 	  g_free(tl->data);
@@ -942,8 +960,8 @@ show_new_slideshow (void)
   gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), spin_button, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (hbox2), start_button, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (hbox2), close_button, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox2), start_button, TRUE, TRUE, 0);
 
   gtk_window_set_default_size (GTK_WINDOW (slideshow_dialog), 1, 1);
 
@@ -990,7 +1008,8 @@ toggle_fullscreen ()
     fullscreen_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
     scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), 
+                                    GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 
     image_widget = gtk_image_new_from_pixbuf (scaled_image_pixbuf);
     image_event_box = gtk_event_box_new ();
@@ -1003,15 +1022,16 @@ toggle_fullscreen ()
     gtk_widget_add_events (GTK_WIDGET (image_event_box), GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 
     gtk_container_add (GTK_CONTAINER (image_event_box), image_widget);
-    //gtk_container_add (GTK_CONTAINER (fullscreen_window), scrolled_window);
+
     gtk_widget_reparent (main_scrolled_window, fullscreen_window);
 
-    gtk_widget_set_size_request (fullscreen_window, gdk_screen_width (), gdk_screen_height ());
+//    gtk_widget_set_size_request (fullscreen_window, gdk_screen_width (), gdk_screen_height ());
     gtk_widget_show_all (fullscreen_window);
 
     gdk_window_fullscreen (GDK_WINDOW (fullscreen_window->window));
 
     fullscreen_toggle_popup = gtk_window_new (GTK_WINDOW_POPUP);
+	
     close_button = gpe_button_new_from_stock (GTK_STOCK_CLOSE, GPE_BUTTON_TYPE_ICON);
     g_signal_connect (G_OBJECT (close_button), "clicked",
 		      G_CALLBACK (toggle_fullscreen), NULL);
@@ -1121,7 +1141,7 @@ main (int argc, char *argv[])
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   main_scrolled_window = scrolled_window;
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+//  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 
   image_widget = gtk_image_new_from_pixbuf (image_pixbuf);
   image_event_box = gtk_event_box_new ();
@@ -1267,7 +1287,8 @@ main (int argc, char *argv[])
   gtk_widget_hide (main_scrolled_window);
     while (gtk_events_pending ())
       gtk_main_iteration ();
-
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(main_scrolled_window), 
+                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   if (argc > 1)
   {
     stat (argv[1], &arg_stat);
